#ifndef HOUGH_TRANSFORM_CIRCLE_H
#define HOUGH_TRANSFORM_CIRCLE_H

#include "ImageOperator.h"
#include "collier_ImageHoughTransform.h"

using namespace csc583Base;

// user fills in one of these for each desired set of search parameters
typedef struct
{
	int	  nSearchID;  // user specified tracking ID
	// set to original size + 2*R to allow for centers outside of original rect
	// set to any smaller size to limit search area
	int   nRows; 	  // size of accumulator 
	int   nCols; 	  // size of accumulator
	int   nPixelScale;// scale for search pixel space
	float fRadius;	  // radius to search for
	float fThresPercent; // accumulator threshold (%max) for circle detection (-1 means don't use)
	float fThresAbsolute; // accumulator threshold for circle detection (-1 means don't use)
} HoughCircleSearch_Type;

typedef struct
{
	HoughCircleSearch_Type Params;
	int					   nPoints; // num pts to "add in" for this circle
	int					   *pnXpts; // precomputed X's for circles
	int					   *pnYpts; // precomputed Y's for circles
	RasterImage_gray_F     *pAccumRasterGF;
} HoughCircleSearchInternal_Type;

// return one of these for each circle found
// may return the same raster to more than one search ID
typedef struct
{
	int   nSearchID;  // user specified tracking ID
	int   nRows; 	  // size of accumulator 
	int   nCols; 	  // size of accumulator
	float fRadius;	  // radius used to perform search
	int   nXc;        // found X center
	int   nYc;		  // found Y center
	float fSum;       // accumulator value at Xc, Yc
	RasterImage_gray_F *pRasterGF; // raster that shows the detection
	
} HoughCircleResults_Type;

namespace csc583ImageProcessing
{

    class CSC583LIB_EXPORT HoughTransform_circle : public ImageHoughTransform
    {
        public:

            /** Defines a zero crossing edge detector
             *
             *  @param  m    dimension of the accumulator array
             *  @param  n[]  resolution of the accumulator along each axis
             */
            HoughTransform_circle(int m, int n[], int nMin, int nMax);
            
			HoughTransform_circle(int nNumSearchs, 
									HoughCircleSearch_Type *pHCSparams);
            
            /** Destructor
             */
            virtual ~HoughTransform_circle(void);


            /** Applies this operator to a RasterImage which is NOT assumed to be binary.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             */ 
            void *applyTo(RasterImage_gray *imgIn);
            
            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the RasterImage received as parameter.
             *
             * This function takes in a gray-level image and a boolean that 
			 * indicates whether that image is in fact a binary image (the 
			 * output of a zero crossing or thresholding operation). It returns 
			 * its results cast as a pointer to void.
			 *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  isBinary    
             */ 
            void *applyTo(RasterImage_gray *imgIn, bool isBinary);


        protected:

        	HoughCircleSearchInternal_Type *pCircleSearch_;
        	HoughCircleResults_Type HCresults_[100];
        
    };
}
#endif  //  HOUGH_TRANSFORM_CIRCLE_H
