#ifndef IMAGE_GRADIENT_GAUSSIAN_H
#define IMAGE_GRADIENT_GAUSSIAN_H

#include "VectorOperator.h"
#include "ImageOperator.h"

using namespace csc583Base;

namespace csc583ImageProcessing
{

    class CSC583LIB_EXPORT ImageGradient_Gaussian : public VectorOperator
    {
        public:

            ImageGradient_Gaussian(double theScale);
            ImageGradient_Gaussian(short theWidth);

            /**
             *  Destructor
             */          
            ~ImageGradient_Gaussian(void);

            /** Applies this operator to an image, returns the result as a VectorField.
             *
             *  Since the operator is a "float" operator, then the VectorField
             *  returned should will be of the "float" type matching the input
             *  image's base raster.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the VectorField obtained by applying the operator to
             *                      imgIn
             */ 

            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**

            VectorField *applyTo(RasterImage *imgIn);

            /** Applies this operator to an image  within the rectangle specified and
             *  returns the result as a VectorField.
             *
             *  The RasterImage returned should be of the "float" form of the input 
             *  image.  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the VectorField obtained by applying the operator to
             *                      imgIn
             */ 

            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**

            VectorField *applyTo(RasterImage *imgIn, ImageRect *theRect);

            /** Applies this operator to a RasterImage and writes the result into a
             *  VectorField received as second parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  vectOut     the VectorField into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, VectorField *vectOut);

            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the VectorField received as parameter.
             *
             *  The RasterImage returned should be of the "float" form of the input 
             *  image.  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  vectOut     the VectorField into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, ImageRect *theRect, VectorField *vectOut);
            
 
        protected:
        
        	double	dSigma_;
        	short	wWidth_;
        	short	wRadius_;
        	float	fNormFactor_;
        	RasterImage_gray_F *DxRasterF_;
        	RasterImage_gray_F *DyRasterF_;
           	RasterImage_gray_F *FilterRasterF_;
	       	RasterImage_gray_F *GaussianRasterF_;
       		RasterImage_gray_F *X_RasterF_;
        	RasterImage_gray_F *Y_RasterF_;
        	
        	VectorField_F *OutVectorFieldF_;
    };
}
#endif  //  IMAGE_GRADIENT_GAUSSIAN_H