/*  NAME:
        Collier_ImageHoughTransform.cpp
 
    DESCRIPTION:
        Base Hough Transform class implementation.
 
    COPYRIGHT:
        csc583VisionLib Copyright  2005, csc583VisionLib Developers.
  
        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2 of the License, or (at your option) any later version.
 
        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.
 
        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation Inc, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    ___________________________________________________________________________
*/

#include <math.h>
//
#include "RasterImage.h"
#include "RasterImage_gray.h"
#include "RasterImage_RGB.h"
#include "RasterImage_RGBa.h"
#include "RasterImage_gray_F.h"

#include "collier_CSC583LIB.h"
#include "collier_ImageMath.h"
#include "collier_ImageHoughTransform.h"


using namespace csc583Base;
using namespace csc583ImageProcessing;


//----------------------------------------------------------
//  static members
//----------------------------------------------------------


//----------------------------------------------------------
//  Constructors and destructors
//----------------------------------------------------------

//
// base constructor for Hough Transform class
//
// This constructor expects a minimal amount of information to set
// up a batch processing string.
//
ImageHoughTransform::ImageHoughTransform(int m, int n[], int nMin, int nMax) 
			       : ImageOperator()
{
	ImageMath	ImageFilter;

	oSearchFixedRange_ = TRUE;
	nNumSizes_ = n[m-1]; // assume last entry is number of accumulators
	nMin_ = nMin;
	nMax_ = nMax;
	if (nNumSizes_ > 1)
	{
		fDeltaDim_ = (nMax_ - nMin_) / (nNumSizes_ - 1);
	}
	else
	{
		fDeltaDim_ = 0;
	}
	

	pnAccumSizes_ = new int[2];
	// just save sizes in case we need them
	pnAccumSizes_[0] = n[0];
	pnAccumSizes_[1] = n[1];
	
	pAccumRastersGF_ = new RasterImage_gray_F*[nNumSizes_];
	for (int i=0; i<nNumSizes_; i++)
	{
		// assume 1st two entries are sizes
		pAccumRastersGF_[i] = new RasterImage_gray_F((short)n[0],(short)n[1]);	
		pAccumRastersGF_[i]->setValidRect(0,0,n[0],n[1]); // reset ValRect for zero fill
		ImageFilter.fillInto(0.0f,pAccumRastersGF_[i]); // zero for edges
	    pAccumRastersGF_[i]->setFloatConversionMode(kRangeFloat);
	}

} // end ImageHoughTransform(m,n,nMin,nMax)

//
// This constructor expects a minimal amount of information to set
// up a batch processing string.
//
ImageHoughTransform::ImageHoughTransform(int nNumSearchs) 
			       : ImageOperator()
{
	oSearchFixedRange_ = FALSE;
	nNumSizes_ = nNumSearchs;
	
	pnAccumSizes_ = NULL;

	pAccumRastersGF_ = new RasterImage_gray_F*[nNumSizes_];
	// must alloc the rasters down in derived class since we don't know the
	// sizes here
	
} // end ImageHoughTransform(nNumSearches)


ImageHoughTransform::~ImageHoughTransform()
{
	if (pnAccumSizes_ != NULL) delete pnAccumSizes_;
	if (pAccumRastersGF_ != NULL) delete []pAccumRastersGF_;

	
} // end ~ImageHoughTransform()

//----------------------------------------------------------
//  application functions
//----------------------------------------------------------

// write stub functions for the virtuals in ImageOperator that we are not using

RasterImage *ImageHoughTransform::applyTo(RasterImage *imgIn)
{
	// declares to prevent warnings
	RasterImage *pImgIn = imgIn;
	
	FAIL_CONDITION(true,
                kMiscError,
                "Function ImageHoughTransform::applyTo(imgIn) not implemented");

	return((RasterImage*)NULL);
}

RasterImage *ImageHoughTransform::applyTo(RasterImage *imgIn, ImageRect *theRect)
{
	// declares to prevent warnings
	RasterImage *pImgIn = imgIn;
	ImageRect   *pRect = theRect;
	
	FAIL_CONDITION(true,
                kMiscError,
                "Function ImageHoughTransform::applyTo(imgIn,theRect) not implemented");

	return((RasterImage*)NULL);
}

void ImageHoughTransform::applyInto(RasterImage *imgIn, RasterImage *imgOut)
{
	// declares to prevent warnings
	RasterImage *pImgIn = imgIn;
	RasterImage *pImgOut = imgOut;

	FAIL_CONDITION(true,
                kMiscError,
                "Function ImageHoughTransform::applyTo(imgIn,imgOut) not implemented");

	return;
}


void ImageHoughTransform::applyInto(RasterImage *imgIn, ImageRect *theRect, RasterImage *imgOut)
{
	// declares to prevent warnings
	RasterImage *pImgIn = imgIn;
	ImageRect   *pRect = theRect;
	RasterImage *pImgOut = imgOut;

	FAIL_CONDITION(true,
                kMiscError,
                "Function ImageHoughTransform::applyTo(imgIn,theRect, imgOut) not implemented");

	return;
}



// only have virtuals which must be overridden
