#ifndef IMAGE_ZERO_CROSSING_H
#define IMAGE_ZERO_CROSSING_H

#include "ImageOperator.h"

using namespace csc583Base;

namespace csc583ImageProcessing
{

    /**	Enumerated type that lists the modes for computing the zero crossing.
	 */
	typedef enum ZeroCrossingMode
	{
			kUndetermined = -1,
			kFull = 0,
			kNeighborsOnly
			
	} ZeroCrossingMode;
	
	class CSC583LIB_EXPORT ImageZeroCrossing : public ImageOperator
    {

        public:

            /** Defines a zero crossing edge detector
             *
             *  @param  theScale    scale parameter sigma of the 2D Gaussian filter
             */
            ImageZeroCrossing(ZeroCrossingMode eMode);

            
            /** Destructor
             */
            virtual ~ImageZeroCrossing(void);

            /** Applies this operator to an image, returns the result as a RasterImage.
             *
             *  The RasterImage returned is of the "float" form of the input image.
             *  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the RasterImage obtained by applying the operator to
             *                      imgIn
             */
             
            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
 
            RasterImage *applyTo(RasterImage *imgIn);

            /** Applies this operator to an image  within the rectangle specified and
             *  returns the result as a RasterImage.
             *
             *  The RasterImage returned is of the "float" form of the input image.
             *  For example, if imgIn was RasterImage_Gray, then the image
             *  returned should be RasterImage_Gray_F.
             *
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @return             the RasterImage obtained by applying the operator to
             *                      imgIn
             */
             
            //**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
			//
			// !!!!! CAVEAT :: USER BEWARE !!!!!
			//
			// Returning a pointer from a function other than "new"
			// 	is never considered a good thing because of the possible
			//	risk of easily generating memory leaks. Therefore,
			//
			// THIS FUNCTION RETURNS A POINTER TO A PROTECTED VARIABLE
			//
			// Each call to this function will delete any prior results
			//	and reinit the internal variable.
			//
			//**//**//**//**//**//**//**//**//**//**//**//**//**//**//**
              
            RasterImage *applyTo(RasterImage *imgIn, ImageRect *theRect);

            /** Applies this operator to a RasterImage and writes the result into a second
             *  RasterImage received as parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  imgOut      the RasterImage into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, RasterImage *imgOut);
            
            /** Applies this operator to a RasterImage within the rectangle specified
             *  and writes the result into the RasterImage received as parameter.
             *
             *  @param  imgIn       the RasterImage to which the operator is applied
             *  @param  theRect     the rectangle within which to apply the filter
             *  @param  imgOut      the RasterImage into which the output is written
             */ 
            void applyInto(RasterImage *imgIn, ImageRect *theRect, RasterImage *imgOut);

        protected:

           	ZeroCrossingMode eCrossMode_;
           	RasterImage_gray_F *fFilterRaster_, *fOutRaster_;

        
    };
}
#endif  //  IMAGE_ZERO_CROSSING_H