/*  NAME:
        Collier_VectorMath.cpp
 
    DESCRIPTION:
        Basic vector field class implementation.
 
    COPYRIGHT:
        csc583VisionLib Copyright  2005, csc583VisionLib Developers.
  
        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2 of the License, or (at your option) any later version.
 
        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.
 
        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation Inc, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
    ___________________________________________________________________________
*/

#include <math.h>
//
#include "RasterImage.h"
#include "RasterImage_gray.h"
#include "RasterImage_RGB.h"
#include "RasterImage_RGBa.h"
#include "RasterImage_gray_F.h"

#include "VectorField.h"
#include "VectorField_F.h"
#include "VectorField_I.h"

#include "collier_CSC583LIB.h"
#include "collier_VectorMath.h"


using namespace csc583Base;
using namespace csc583ImageProcessing;


//----------------------------------------------------------
//  static members
//----------------------------------------------------------


//----------------------------------------------------------
//  Constructors and destructors
//----------------------------------------------------------

VectorMath::VectorMath(void)
{
} // end VectorMath(void)


VectorMath::~VectorMath()
{
} // end ~VectorMath()

//----------------------------------------------------------
//  application functions
//----------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////
//
// Divergence (x+y) function
//
void VectorMath::getDivergence(VectorField *vectIn, RasterImage *imgOut)
{
     // make sure output image is larger than input vector          
    FAIL_CONDITION(imgOut->getHeight() < vectIn->getHeight(),
                kRasterImageError,
                "Output image has less rows than input vector in getDivergence(A,Out)");
    FAIL_CONDITION(imgOut->getWidth() < vectIn->getWidth(),
                kRasterImageError,
                "Output image has less columns than input vector in getDivergence(A,Out)");


	// make sure we have a gray level image and not color
	FAIL_CONDITION(imgOut->getBaseRasterType() != kGrayRaster,
                kRasterImageError,
                "Only gray-level output rasters accepted in getDivergence(A,Out)");


	// get limits of operation based on valid image (A) size and filter (B) size
	ImageRect *pValidRectIn   = vectIn->getValidRect();
	
	int nInXo   = pValidRectIn->getLeft(),
		nInXn   = pValidRectIn->getRight(),
		nInYo   = pValidRectIn->getTop(),
		nInYn   = pValidRectIn->getBottom();
                
	// for all floats
	 if (vectIn->hasFloatRaster() && imgOut->hasFloatRaster())
	 {
		float **fOut   = ((RasterImage_gray_F*)imgOut)->getGrayF2D();
		float **fXvect = (float **)vectIn->getRasterX2D();
		float **fYvect = (float **)vectIn->getRasterY2D();
	
		for (int i=nInYo; i<=nInYn; i++)
		{
			for (int j=nInXo; j<=nInXn; j++)
			{
				fOut[i][j] = fXvect[i][j] + fYvect[i][j];
			}
		}
	 }
	 // for all unsigned chars
	 else if (!vectIn->hasFloatRaster() && !imgOut->hasFloatRaster())
	 {
		unsigned char **ubOut 	= ((RasterImage_gray*)imgOut)->getGray2D();
		unsigned char **ubXvect = (unsigned char **)vectIn->getRasterX2D();
		unsigned char **ubYvect = (unsigned char **)vectIn->getRasterY2D();
	
		for (int i=nInYo; i<=nInYn; i++)
		{
			for (int j=nInXo; j<=nInXn; j++)
			{
				ubOut[i][j] = ubXvect[i][j] + ubYvect[i][j];
			}
		}
	 }
	 else
	 {
	 	// mismatched types, FAIL for now
		FAIL_CONDITION(TRUE,
                kRasterImageError,
                "Only same typed rasters accepted in convolve(A,B,Out)");
     }
     
     // now set the output valid rectangle
     imgOut->setValidRect(vectIn->getValidRect());
	 	
} // end VectorMath::getDivergence(VectorField *imgA, RasterImage *imgOut)
